#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include "librf625.h"


#ifdef __linux__
#include <arpa/inet.h>
#include <curses.h>
#else
#include <conio.h>
#endif

std::map<int,std::ofstream*> dump_files;

void udp_proc(rf625_profile prf)
{
    /* Here we have all neccessary data in prf structure
     * let's write measure and packet counters and number of points to output file */
    std::ofstream *f = dump_files[prf.serial_number];
    *f << prf.measure_cnt << "\t" << prf.packet_cnt << "\t" << prf.points.size() << std::endl;
}

int main(int argc, char *argv[])
{
    /* Initialize librf625 */
    rf625::init();

    /* Execute search for scanners */
    std::cout << "Searching..." << std::endl;
    rf625_list dev_list = rf625::search(3000);
    std::cout << std::endl << "#\t| S/N\t| Address\t| Port" << std::endl;
    if (dev_list.empty()) {
        std::cout << "Nothing found" << std::endl;
        return 1;
    }

    /* List scanners info
     * create output stream for each scanner */
    for(int i=0; i<dev_list.size(); i++)
    {
        auto dev = dev_list.at(i);
        if (!dev) continue;
        in_addr addr;
        uint32_t sn = dev->get_info().serial_number;
        addr.s_addr = dev->get_info().ip_address;
        std::cout << i << "\t| "
                  << sn << "\t| "
                  << inet_ntoa(addr) << "\t| "
                  << dev->get_info().udp_port << std::endl;
        std::ofstream *f = new std::ofstream();
        std::string fname = "dmp_" + std::to_string(sn) + ".txt";
        f->open(fname.data(), std::ofstream::out | std::ofstream::trunc);
        dump_files[sn] = f;
    }

    /* Start UDP streaming for each scanner in new thread */
    std::cout << std::endl << "Press Esc to stop" << std::endl;
    std::cout << std::flush;
    char c;
    for (auto dev: dev_list) {
        dev->udp_register_callback(udp_proc);
    }
    while (true)
    {
        c = getch();
        if (c==27)
            break;  /* Break if Esc key pressed */
    }

    /* Stop threads by unregistering callback function
     * note you can switch between different callback functions without unregistering them */
    for (auto dev: dev_list) {
        dev->udp_unregister_callback();
    }

    /* Cleanup librf625 */
    rf625::cleanup();
    /* close output streams and free memory */
    for (auto pair: dump_files) {
        std::ofstream *f = pair.second;
        f->close();
        delete f;
    }
    return 0;
}
